#include <stdio.h>
#include <Application.h>
#include <Window.h>
#include <View.h>
#include <Rect.h>

enum {
	updateMsg = 1,
};

class TestView: public BView
{
public:
	TestView(BRect frame, const char *name): BView(frame, name, B_FOLLOW_NONE, B_FULL_UPDATE_ON_RESIZE | B_WILL_DRAW | B_SUBPIXEL_PRECISE)
	{
		this->SetViewColor(B_TRANSPARENT_COLOR);
	}

	void Draw(BRect dirty)
	{
		BRect rect(BPoint(0, 0), this->Frame().Size());
		int i = 0;
		this->PushState();
		while ((rect.Width() > 0) && (rect.Height() > 0)) {
			switch (i) {
			case 0: this->SetHighColor(0xff, 0x00, 0x00); break;
			case 1: this->SetHighColor(0x00, 0xff, 0x00); break;
			case 2: this->SetHighColor(0x00, 0x00, 0xff); break;
			}
			this->FillRect(rect, B_SOLID_HIGH);
			this->Sync();
			snooze(10000);
			rect.InsetBy(4, 4);
			i = (i + 1) % 3;
		}
		this->PopState();
		Looper()->PostMessage(updateMsg, this);
	}

	void MessageReceived(BMessage *msg)
	{
		switch(msg->what) {
		case updateMsg: {
			Invalidate();
			break;
		}
		default:
			BView::MessageReceived(msg);
		}
	}

};

class TestWindow: public BWindow
{
private:
	TestView *fView;
public:
	TestWindow(BRect frame): BWindow(frame, "TestApp", B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS)
	{
		fView = new TestView(BRect(BPoint(0, 0), frame.Size()), "view");
		fView->SetResizingMode(B_FOLLOW_ALL);
		AddChild(fView, NULL);
		fView->MakeFocus(true);
	}

};

class TestApplication: public BApplication
{
public:
	TestApplication(): BApplication("application/x-vnd.test-app")
	{
	}

	void ReadyToRun() {
		BWindow *wnd = new TestWindow(BRect(32, 32, 256, 256));
		wnd->SetFlags(wnd->Flags() | B_QUIT_ON_WINDOW_CLOSE);
		wnd->Show();
	}
};


int main()
{
	TestApplication app;
	app.Run();
	return 0;
}
